////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// NDK 2.0 - Network Development Kit                                          //
//                                                                            //
// Authors: Sebastien Lachance                                                //
//                                                                            //
// E-mail:  netblitz@rocler.qc.ca                                             //
//                                                                            //
// -------------------------------------------------------------------------- //
//                                                                            //
// Permission to use, copy, modify, and distribute this software for any      //
// purpose and without fee is hereby granted. This is no guarantee about the  //
// use of this software. For any comments, bugs or thanks, please email us.   //
//                                                                            //
// -------------------------------------------------------------------------- //
//                                                                            //
// Targeted Platform: Any Windows version                                     //
//                                                                            //
// Compiled with:     Visual C++ 6.0                                          //
//                                                                            //
// Last modification: January 2002                                            //
//                                                                            //
// History:                                                                   //
//                                                                            //
// 1- First release of this file.                                             //
// 2- The class is renamed and some optimizations are applied. Hungarian      //
//    notation is used.                                                       //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Includes                                                                   //
////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "NDKUserManager.h"
#include "NDKServer.h"


////////////////////////////////////////////////////////////////////////////////
// Constructors / Destructor                                                  //
////////////////////////////////////////////////////////////////////////////////

// Constructor.
CNDKUserManager::CNDKUserManager()
{
	m_lNextId = 0;
}


// Destructor.
CNDKUserManager::~CNDKUserManager()
{
	RemoveAllUsers();
}


////////////////////////////////////////////////////////////////////////////////
// Attributes                                                                 //
////////////////////////////////////////////////////////////////////////////////

// Gets the number of users.
long CNDKUserManager::GetNbUsers() const
{
	return m_users.GetCount();
}


// Returns the Ids of all users.
void CNDKUserManager::GetUserIds(CLongArray& alIds) const
{
	alIds.RemoveAll();

	POSITION pos = m_users.GetHeadPosition();

	while (pos != NULL)
	{
		CNDKUser& user = m_users.GetNext(pos);
		alIds.Add(user.GetId());
	}
}


//IQ_egal_0 modifications START

//Returns the extended data
DWORD CNDKUserManager::GetExData(long lUserId)
{
	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
		return user.GetExData();
	return 0;
}

//Sets the extended data, returning the old one
DWORD CNDKUserManager::SetExData(long lUserId, DWORD dwExData)
{
	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
	{
		POSITION pos = m_users.Find(user);
		if (pos != NULL)
		{
			DWORD dwOldExData = user.GetExData();
			user.SetExData(dwExData);
			m_users.SetAt(pos, user);
			return dwOldExData;
		}
	}
	return 0;
}

//Returns the send state
BOOL CNDKUserManager::GetSendState(long lUserId)
{
	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
		return user.GetSendState();
	return TRUE;
}

//Sets the send state, returning the old one
BOOL CNDKUserManager::SetSendState(long lUserId, BOOL bCanSend)
{
	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
	{
		POSITION pos = m_users.Find(user);
		if (pos != NULL)
		{
			BOOL bOldCanSend = user.GetSendState();
			user.SetSendState(bCanSend);
			m_users.SetAt(pos, user);
			return bOldCanSend;
		}
	}
	return TRUE;
}
	
//Returns the wait for reply state
BOOL CNDKUserManager::GetWaitForReplyState(long lUserId)
{
	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
		return user.GetWaitForReplyState();
	return FALSE;
}

//Sets the wait for reply state, returning the old one
BOOL CNDKUserManager::SetWaitForReplyState(long lUserId, BOOL bWaitForReply)
{
	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
	{
		POSITION pos = m_users.Find(user);
		if (pos != NULL)
		{
			BOOL bOldWaitForReply = user.GetWaitForReplyState();
			user.SetWaitForReplyState(bWaitForReply);
			m_users.SetAt(pos, user);
			return bOldWaitForReply;
		}
	}
	return FALSE;
}

//IQ_egal_0 modifications stop

////////////////////////////////////////////////////////////////////////////////
// Operations                                                                 //
////////////////////////////////////////////////////////////////////////////////

// Adds a user then returns its new Id.
long CNDKUserManager::AddUser(CNDKServerSocket* pServerSocket)
{
	long lNextId = 1;

	POSITION pos = m_users.GetHeadPosition();
	while( pos )
	{
		if( m_users.GetNext( pos ).GetId() == lNextId )
		{
			lNextId++;
			pos = m_users.GetHeadPosition();
		}
	}

	m_users.AddTail(CNDKUser(lNextId, pServerSocket));
	
	return lNextId;
}


// Removes a user.
BOOL CNDKUserManager::RemoveUser(long lUserId)
{
	BOOL bResult = FALSE;

	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
	{
		POSITION pos = m_users.Find(user);

		if (pos != NULL)
		{
			user.CloseSocket();
			m_users.RemoveAt(pos);
			bResult = TRUE;
		}
	}

	return bResult;
}


// Removes all users.
void CNDKUserManager::RemoveAllUsers()
{
	POSITION pos = m_users.GetHeadPosition();

	while (pos != NULL)
	{
		CNDKUser& user = m_users.GetNext(pos);
		user.CloseSocket();
	}

	m_users.RemoveAll();
}


// Sends a message to a specified user.
BOOL CNDKUserManager::SendMessage(long lUserId, CNDKMessage &msg)
{
	BOOL      bResult = FALSE;
	CNDKUser& user = GetUserFromId(lUserId);

	if (user.GetId() > 0)
	{	
		bResult = user.SendMessage(msg);

		if (!bResult)
			RemoveUser(user.GetId());
	}

	return bResult;
}


// Sends a message to all users.
BOOL CNDKUserManager::SendMessageToAllUsers(CNDKMessage& message, CLongList& listIds)
{
	POSITION pos = m_users.GetHeadPosition();

	while (pos != NULL)
	{
		CNDKUser& user = m_users.GetNext(pos);
		
		if (!user.SendMessage(message))
			listIds.AddTail(user.GetId());
	}

	BOOL bResult = listIds.IsEmpty();

	pos = listIds.GetHeadPosition();
	
	while (pos != NULL)
		RemoveUser(listIds.GetNext(pos));

	return bResult;
}


// Sends a message to all users except for user specified in alUserIds.
BOOL CNDKUserManager::SendMessageToAllUsersExceptFor(const CLongArray& alUserIds, 
													 CNDKMessage& message, 
													 CLongList& listIds)
{
	POSITION pos = m_users.GetHeadPosition();

	while (pos != NULL)
	{
		CNDKUser& user = m_users.GetNext(pos);

		BOOL bFound = FALSE;

		for (long lUserIndex = 0; lUserIndex < alUserIds.GetSize(); lUserIndex++)
		{
			if (alUserIds[lUserIndex] == user.GetId())
			{
				bFound = TRUE;
				break;
			}
		}
		
		// Send the message only if the user isn't in the array of exception
		if (!bFound && !user.SendMessage(message))
			listIds.AddTail(user.GetId());
	}

	BOOL bResult = listIds.IsEmpty();

	pos = listIds.GetHeadPosition();
	
	while (pos != NULL)
		RemoveUser(listIds.GetNext(pos));

	return bResult;

}


// Processes pending read.
BOOL CNDKUserManager::ProcessPendingRead(CNDKServerSocket* pSocket, long lErrorCode, 
										  long& lUserId, CMessageList& messages)
{
	BOOL      bResult = FALSE;
	CNDKUser& user = GetUserFromSocket(pSocket);

	lUserId = user.GetId();

	if ((lErrorCode == 0) && (lUserId > 0))
	{
		do
		{
			CNDKMessage message;
			bResult = user.ReadMessage(message);

			messages.AddTail(message);

			// m_users.Find(user) is needed to check if the user is still 
			// connected because if the OnMessage causes a disconnection of 
			// the user, we must stop the reading.
		} while (bResult && m_users.Find(user) && !user.IsSocketBufferEmpty());
	}

	return bResult;
}


////////////////////////////////////////////////////////////////////////////////
// Private Operations                                                         //
////////////////////////////////////////////////////////////////////////////////

// Gets a user from an Id.
CNDKUser CNDKUserManager::GetUserFromId(long lUserId) const
{
	CNDKUser user;
	POSITION pos = m_users.GetHeadPosition();

	while (pos != NULL)
	{
		user = m_users.GetNext(pos);

		if (user.GetId() == lUserId)
			pos = NULL;
	}

	return user;
}


// Gets a user from a socket.
CNDKUser CNDKUserManager::GetUserFromSocket(CNDKServerSocket* pServerSocket) const
{
	CNDKUser user;
	POSITION pos = m_users.GetHeadPosition();

	while (pos != NULL)
	{
		user = m_users.GetNext(pos);

		if (user.IsSocketEqual(pServerSocket))
			pos = NULL;
	}

	return user;
}
